/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.unomi.services.conditions.initializers;

import org.apache.unomi.api.conditions.initializers.ChoiceListInitializer;
import org.apache.unomi.api.conditions.initializers.ChoiceListValue;
import org.apache.unomi.api.conditions.initializers.I18nSupport;
import org.apache.unomi.api.services.EventService;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class EventTypeIdChoiceListInitializer implements ChoiceListInitializer, I18nSupport {

    EventService eventService;

    public void setEventService(EventService eventService) {
        this.eventService = eventService;
    }

    public List<ChoiceListValue> getValues(Object context) {
        List<ChoiceListValue> choiceListValues = new ArrayList<ChoiceListValue>();
        Set<String> eventTypeIds = eventService.getEventTypeIds();
        for (String eventProperty : eventTypeIds) {
            String resourceKey = "eventType." + eventProperty;
            choiceListValues.add(new ChoiceListValue(eventProperty, resourceKey));
        }
        return choiceListValues;
    }
}
